#include <SPI.h>
#include "ozbot.h"			//設定OZBOT

#define MaxSpeedR 900	 //右輪最大值
#define MaxSpeedL 900	 //左輪最大值
#define MinSpeedR 0 	 //右輪最小值
#define MinSpeedL 0		 //左輪最小值
#define AccDis    100	 //可加速距離
#define DecDis    30	 //要減速距離
#define CENSpeed  300  //基礎速度

//定義按鈕腳位
#define Button_1  9
#define Button_2  8

unsigned char Distance[4];
unsigned char SonarFlag;

int iTACHL, iTACHR, ACCT_L, ACCT_R, iNowSpeed, iSpeedL, iSpeedR, Err;

//定義 校正時使用速度
#define	CAL_DZ_SPEED_L	230
#define CAL_DZ_SPEED_R	230

int iSecLenL, iSecLenR;
int DeadZoneL, DeadZoneR;

void setup() {

  pinMode(Button_1, INPUT);
  digitalWrite(Button_1, HIGH);
  pinMode(Button_2, INPUT);
  digitalWrite(Button_2, HIGH);  
  OzBot.RepeatRanging() ;
  OzBot.GetMotorDeadZone(DeadZoneL, DeadZoneR);
}
void loop() {

  if (digitalRead(Button_1) == LOW) {
    CalDeadZone();
  }
  if (digitalRead(Button_2) == LOW) {
    while (1)
      ForwardWithSonar();
  }
}


void CalDeadZone() {

  OzBot.Beep();	//啟動蜂鳴器
  delay(1000);
  //取得校正速度DeadZone
  OzBot.GetMotorDeadZone(DeadZoneL, DeadZoneR);
  //設定速度，將初始速度加入校正速度
  iSecLenL = CAL_DZ_SPEED_L + DeadZoneL;
  iSecLenR = CAL_DZ_SPEED_R + DeadZoneR;
  //前進2秒，並取得左右輪差
  OzBot.ForwardLR(iSecLenL, iSecLenR);
  delay(500);
  OzBot.SetTachInLR(0, 0);
  delay(2000);
  OzBot.TachInDual(iSecLenL, iSecLenR);
  OzBot.StopDual();

  //依照左右輪差值，調整校正速度DeadZone
  iSecLenR = (iSecLenL - iSecLenR) / 16;
  if (iSecLenR > 80) {
    iSecLenR = 80 ;
  }
  else if (iSecLenR < -80) {
    iSecLenR = -80;
  }
  if (iSecLenR >= 0) {
    DeadZoneR = DeadZoneR + iSecLenR;
  }
  else {
    DeadZoneL = DeadZoneL - iSecLenR;
  }
  if (DeadZoneL >= DeadZoneR) {
    DeadZoneL = DeadZoneL - DeadZoneR;
    DeadZoneR = 0;
  }
  else {
    DeadZoneR = DeadZoneR - DeadZoneL;
    DeadZoneL = 0;
  }
  //將校正速度DeadZone存入記憶體中
  OzBot.SetMotorDeadZone(DeadZoneL, DeadZoneR);
}

void CheckSonarErr() {
  unsigned char i;
  unsigned char SonarErr[5] = {30, 20, 20, 20, 30};
  unsigned char Err[3]  = {1, 2, 4};
  OzBot.GetDistance(Distance[0], Distance[1], Distance[2], Distance[3], Distance[4]);
  SonarFlag = 0;
  for (i = 1 ; i <= 3 ; i++) {
    if (Distance[i] < SonarErr[i]) {
      SonarFlag = SonarFlag + Err[i - 1];
    }
  }
  delay(100);
}

// 取得左右輪差，並記錄行駛距離

void GetErr() {
  OzBot.TachInDual(iTACHL, iTACHR);
  OzBot.SetTachInLR(0, 0);
  ACCT_L = ACCT_L + iTACHL;
  ACCT_R = ACCT_R + iTACHR;
  Err = (ACCT_L - ACCT_R) * 3;
  if (Err >  iNowSpeed / 2) Err = iNowSpeed / 2;
  if (Err < -iNowSpeed / 2) Err = -iNowSpeed / 2;
  delay(10);
}

void Forward(int Spd)  {
  GetErr();
  iSpeedL = Spd - Err + DeadZoneL;
  iSpeedR = Spd + Err + DeadZoneR;

  if (iSpeedL > MaxSpeedL) iSpeedL = MaxSpeedL;
  if (iSpeedL < MinSpeedL) iSpeedL = MinSpeedL;
  if (iSpeedR > MaxSpeedR) iSpeedR = MaxSpeedR;
  if (iSpeedR < MinSpeedR) iSpeedR = MinSpeedR;

  OzBot.SetVelLR(iSpeedL, iSpeedR);
}

void ForwardWithSonar() {
  CheckSonarErr();
  if (SonarFlag != 0) {
    OzBot.StopDual();
  }
  else {
    iNowSpeed = CENSpeed;
    Forward(iNowSpeed);
  }
}
