#include <SPI.h>
#include "ozbot.h"

#define MaxSpeedR 900	  //右輪最大值
#define MaxSpeedL 900	  //左輪最大值
#define MinSpeedR 0 	  //右輪最小值
#define MinSpeedL 0	  //左輪最小值
#define AccDis    40	  //可加速距離
#define CENSpeed  300     //基礎速度
#define MAXMoveSpd 300
#define MinMoveSpd 250
#define ForwardSpdMax 450
#define ForwardSpdMin 250
#define bForwardStep 20

#define iInBackBlockDis 	400     //進入黑色區堿深度
#define iBackBlockTurnDis	550	//調頭旋轉距離
#define bTurnStep               3	//轉彎級距
#define TurnDisL	460
#define TurnDisR	450
//定義按鈕腳位
#define Button_1        9
#define Button_2        8

unsigned char Distance[5];
unsigned char  SonarFlag;
unsigned char ForwardDis;

int iTACHL, iTACHR, ACCT_L, ACCT_R, iNowSpeed, iSpeedL, iSpeedR, Err, iTargetspeed;

//定義 校正時使用速度
#define	CAL_DZ_SPEED_L	230
#define CAL_DZ_SPEED_R	230

int iSecLenL, iSecLenR;
int DeadZoneL, DeadZoneR;
unsigned char bIR;
void setup() {
  pinMode(Button_1, INPUT);
  pinMode(Button_2, INPUT);
  digitalWrite(Button_1, HIGH);
  digitalWrite(Button_2, HIGH);
  delay(500);
  OzBot.Beep();
  OzBot.RepeatRanging();
  OzBot.GetMotorDeadZone(DeadZoneL, DeadZoneR);
  iNowSpeed = ForwardSpdMin;
}
void loop() {
  if (digitalRead(Button_1) == LOW) {
    CalDeadZone();
  }
  if (digitalRead(Button_2) == LOW) {
    OzBot.Beep();
    delay(1000);
    do {
      ForwardWithSonar();
      OzBot.GetIr(bIR);
    } while (bIR != 31);
    do {
      ForwardWithSonar();
      OzBot.GetIr(bIR);
    } while (bIR != 0);
    FindBackLine();
    OzBot.GetIr(bIR);
    do {
      ForwardWithSonar();
      OzBot.GetIr(bIR);
    } while (bIR != 31);
    do {
      ForwardWithSonar();
      OzBot.GetIr(bIR);
    } while (bIR != 0);
    FindBackLine();
  }
}


void CalDeadZone(void) {

  OzBot.Beep();	//啟動蜂鳴器
  delay(1000);
  //取得校正速度DeadZone
  OzBot.GetMotorDeadZone(DeadZoneL, DeadZoneR);
  //設定速度，將初始速度加入校正速度
  iSecLenL = CAL_DZ_SPEED_L + DeadZoneL;
  iSecLenR = CAL_DZ_SPEED_R + DeadZoneR;
  //前進2秒，並取得左右輪差
  OzBot.ForwardLR(iSecLenL, iSecLenR);
  delay(500);
  OzBot.SetTachInLR(0, 0);
  delay(2000);
  OzBot.TachInDual(iSecLenL, iSecLenR);
  OzBot.StopDual();

  //依照左右輪差值，調整校正速度DeadZone
  iSecLenR = (iSecLenL - iSecLenR) / 16;
  if (iSecLenR > 80 ) {
    iSecLenR = 80;
  }
  else if (iSecLenR < -80 ) {
    iSecLenR = -80;
  }
  if (iSecLenR >= 0 ) {
    DeadZoneR = DeadZoneR + iSecLenR;
  }
  else {
    DeadZoneL = DeadZoneL - iSecLenR;
  }
  if (DeadZoneL >= DeadZoneR ) {
    DeadZoneL = DeadZoneL - DeadZoneR;
    DeadZoneR = 0;
  }
  else {
    DeadZoneR = DeadZoneR - DeadZoneL;
    DeadZoneL = 0;
  }
  //將校正速度DeadZone存入記憶體中
  OzBot.SetMotorDeadZone(DeadZoneL, DeadZoneR);
}

void CheckSonarErr(void) {
  unsigned char i;
  unsigned char SonarErr[] = {30, 30, 30, 30, 30};
  unsigned char ErrSon[] = {1, 2, 4};
  OzBot.GetDistance(Distance[0], Distance[1], Distance[2], Distance[3], Distance[4]);
  SonarFlag = 0;
  for (i = 1; i <= 3 ; i++) {
    if (Distance[i] == 0 ) {
      Distance[i] = 255;
    }
    if (Distance[i] < SonarErr[i] ) {
      SonarFlag = SonarFlag + ErrSon[i - 1] ;
    }
  }
  if (Distance[1] < Distance[2] ) {
    ForwardDis = Distance[1];
  }
  else {
    ForwardDis = Distance[2];
  }

  if (ForwardDis > Distance[3] ) {
    ForwardDis = Distance[3];
  }

  delay(100);

}

//取得左右輪差，並記錄行駛距離

void GetErr() {
  OzBot.TachInDual(iTACHL, iTACHR);
  if ((iTACHL > 30000) || (iTACHR > 30000))  {
    OzBot.SetTachInLR(0, 0);
  }
  Err = (iTACHL - iTACHR) * 3;
  if (Err > (iNowSpeed / 2)) {
    Err = iNowSpeed / 2;
  }
  else if (Err < (-iNowSpeed / 2)) {
    Err = -iNowSpeed / 2;
  }
}

void Forward(int Spd) {
  GetErr();
  iSpeedL = Spd - Err + DeadZoneL;
  iSpeedR = Spd + Err + DeadZoneR;

  if (iSpeedL > MaxSpeedL)      iSpeedL = MaxSpeedL;
  else if (iSpeedL < MinSpeedL) iSpeedL = MinSpeedL;
  if (iSpeedR > MaxSpeedR)      iSpeedR = MaxSpeedR;
  else if (iSpeedR < MinSpeedR) iSpeedR = MinSpeedR;
  OzBot.SetVelLR(iSpeedL, iSpeedR);
}

void FindBackLine() {
  OzBot.SetTachInLR(0, 0);
  do {
    Forward(CENSpeed);
  } while (iTACHR <= iInBackBlockDis);
  OzBot.BrakeDual();
  delay(100);
  MoveWithSpd(3, bTurnStep, iBackBlockTurnDis);
  OzBot.SetTachInLR(0, 0);
}

void ForwardWithSonar() {
  CheckSonarErr();
  if (SonarFlag != 0 ) {
    //		OzBot.StopDual()
    Avoid();
  }
  else {
    if (ForwardDis > AccDis ) {
      iNowSpeed = iNowSpeed + bForwardStep;
      if (iNowSpeed > ForwardSpdMax ) {
        iNowSpeed = ForwardSpdMax;
      }
    }
    else {
      iNowSpeed = ForwardSpdMin;
    }
    Forward(iNowSpeed);
  }
}
void Avoid() {
  unsigned int TurnDis;
  //                                 000,001,010,011,100,101,110,111
  const unsigned int TurnForwardDisL[] = {  0, 150, 300, 150, 450, 300, 300, 300};
  const unsigned int TurnForwardDisR[] = {  0, 450, 300, 300, 150, 300, 150, 300};
  CheckSonarErr();
  if (SonarFlag != 0 ) {
    OzBot.Beep();
    if (Distance[0] >= Distance[4] ) {
      OzBot.Beep();
      TurnDis = TurnForwardDisR[SonarFlag];
      MoveWithSpd(2, bTurnStep, TurnDisR);
      MoveWithSpd(0, bTurnStep, TurnDis);
      MoveWithSpd(1, bTurnStep, TurnDisL);
    }
    else {
      OzBot.Beep();
      TurnDis = TurnForwardDisL[SonarFlag];
      MoveWithSpd(1, bTurnStep, TurnDisL);
      MoveWithSpd(0, bTurnStep, TurnDis);
      MoveWithSpd(2, bTurnStep, TurnDisR);
    }
  }
}

void DisplaySonarFlag() {
  unsigned char bDisplay, bCnt;
  OzBot.CursorRC(2, 1);	//將游標移至第2行開頭

  for (bCnt = 0 ;  bCnt < 5 ; bCnt++) {
    bDisplay = SonarFlag & 1;
    SonarFlag = SonarFlag >> 1;
    OzBot.Display(bDisplay);
  }
}


//緩加減速 bDir 0:前進 1:左轉 2:右轉 ,iStep 緩加減速級距，iDisCnt 距離

void MoveWithSpd(unsigned char bDir, int iStep, int iDisCnt) {
  int iMoveSpd = 0;
  OzBot.SetTachInLR(0, 0);
  do {
    OzBot.TachInDual(iTACHL, iTACHR);

    if (bDir == 1 ) {
      iTACHL = iTACHR;
    }

    if (iTACHL < (iDisCnt / 2) ) {
      if (iMoveSpd < MAXMoveSpd) {
        iMoveSpd = iMoveSpd + iStep;
        if (iMoveSpd > MAXMoveSpd)  {
          iMoveSpd = MAXMoveSpd;
        }
        switch (bDir) {
          case 0: Forward(iMoveSpd); break;
          case 1: OzBot.SetVelLR(0, iMoveSpd); break;
          case 2: OzBot.SetVelLR(iMoveSpd, 0); break;
          case 3: OzBot.SetVelLR(iMoveSpd, -iMoveSpd); break;
        }
      }
    }
  } while (iTACHL < (iDisCnt / 2));

  do {
    OzBot.TachInDual(iTACHL, iTACHR);

    if (bDir == 1 ) {
      iTACHL = iTACHR;
    }

    if (iTACHL < iDisCnt)  {
      if (iMoveSpd > MinMoveSpd) {
        iMoveSpd = iMoveSpd - iStep * 3;
        if (iMoveSpd < MinMoveSpd) {
          iMoveSpd = MinMoveSpd;
        }
        switch (bDir) {
          case 0: Forward(iMoveSpd); break;
          case 1: OzBot.SetVelLR(0, iMoveSpd); break;
          case 2: OzBot.SetVelLR(iMoveSpd, 0); break;
          case 3: OzBot.SetVelLR(iMoveSpd, -iMoveSpd); break;
        }
      }
    }
  } while (iTACHL < iDisCnt);
  iNowSpeed = ForwardSpdMin;
  OzBot.BrakeDual();
  delay(100);
  OzBot.SetTachInLR(0, 0);
}
