/* 使用說明
 1. 記錄圈 - 放在線上, 按右鍵(16), 聽到嗶聲立即放開
 2. 競速圈 - 放在線上, 按左鍵(17)
 3. 校正 IR - 放在線上, 按住右鍵(16), 聽到嗶第兩聲立即放開
 4. 校正左右輪差 - 按住右鍵(16), 聽到嗶第三聲立即放開, 請重覆此步驟直到車子約略呈直線行駛
 */
#include <SPI.h>
#include "ozbot.h"

#define BRAKE_TIME		5
#define CROSS_COUNT		129
#define Rot_Time		30

uint16_t bAccFreq, bStopFreq;
#define bAccPower       5
#define bStopPower      5
#define CAL_DZ_SPEED_L 230
#define CAL_DZ_SPEED_R 230

//------------------------------------------------------------------------
//	M1模組內部PID參數值 (0 ~ 255)
//------------------------------------------------------------------------
#define PID_VAL_P		18
#define PID_VAL_I		0
#define PID_VAL_D		40

#define PID_VAL_P_Acc	25
#define PID_VAL_I_Acc	0
#define PID_VAL_D_Acc	245

#define PID_SCALAR		0
/*------------------------------------------------------------------------
 	模組內部最高最低速限，以及直線速度 (-1024 ~ 1024)
 ------------------------------------------------------------------------*/
#define	CAL_SPEED_L	 (170)
#define	CAL_SPEED_R	 (170)

#define	REC_SPEED_L	 (240)
#define	REC_SPEED_R	 (240)

#define	ROT_SPEED_L      (460)
#define	ROT_SPEED_R	 (460)

#define	ROT_SPEED_L_Acc  (460)
#define	ROT_SPEED_R_Acc	 (460)

#define CEN_SPEED_L	 (400)
#define	CEN_SPEED_R	 (400)

#define	MAX_SPEED_L		1024
#define MAX_SPEED_R		1024
#define	MIN_SPEED_L		-1024
#define MIN_SPEED_R		-1024
/*------------------------------------------------------------------------
 	直線加速各項設定值
 ------------------------------------------------------------------------*/
#define ACC_Max			900
#define HOLD_STOP		2

/*------------------------------------------------------------------------
 '	模組各別感測階段的誤差值 (0 ~ 127)
 '------------------------------------------------------------------------*/
#define ERROR_VAL_1	1
#define ERROR_VAL_2	2
#define ERROR_VAL_3	4
#define ERROR_VAL_4	8

unsigned char I, Cnt, bIR;
unsigned char Point, Crossroad, Direction;
unsigned char bCnt, bNxtCnt;
unsigned char bStatus, bPassFlag;
int16_t sTargetSpeed, sAccSpeedL, sAccSpeedR;
int16_t iSecLenL, iSecLenR, iStopLen, iStopLen2, iTotalLen, iTargetCntFlag, iStopTargetLen;
unsigned char iChar;
int16_t DeadZoneL, DeadZoneR;

void CheckNext(void)
{
  OzBot.GetSecDir(bNxtCnt, Direction);
  OzBot.GetSecLen(bNxtCnt, iTotalLen, iSecLenR);
  bPassFlag = bNxtCnt;
  while (Direction == 2) {
    bPassFlag = bPassFlag + 1;
    OzBot.GetSecLen(bPassFlag, iSecLenL, iSecLenR);
    OzBot.GetSecDir(bPassFlag, Direction);
    iTotalLen += iSecLenL;
  }
  //////速度與剎車計算//////  
  sTargetSpeed = (iTotalLen / 4) + CEN_SPEED_L;

  if (sTargetSpeed > ACC_Max) {
    sTargetSpeed = ACC_Max;
  }

  if(sTargetSpeed > 500) {
    OzBot.SetP(PID_VAL_P_Acc);
    OzBot.SetI(PID_VAL_I_Acc);
    OzBot.SetD(PID_VAL_D_Acc);
    bAccFreq = sTargetSpeed/400;
    bStopFreq = sTargetSpeed/1000;
    iStopLen = iTotalLen - (iTotalLen/6) - 600;
  }
  else {
    OzBot.SetP(PID_VAL_P);
    OzBot.SetI(PID_VAL_I);
    OzBot.SetD(PID_VAL_D);
    bAccFreq = sTargetSpeed/1000;
    bStopFreq = sTargetSpeed/1000;
    iStopLen = iTotalLen - 150;
  }
  iStopLen2 = iTotalLen - 150;
}
/*------------------------------------------------------------------------
 	讀取左輪行走距離直到超過設定值
 ------------------------------------------------------------------------*/
void CheckCount(int16_t iTarCntL)
{
  int16_t iCurCntL, iCurCntR;

  iTargetCntFlag = 0;
  sAccSpeedL = CEN_SPEED_L;
  sAccSpeedR = CEN_SPEED_R;
  do {
    OzBot.GetCurSecTACH(iCurCntL, iCurCntR);
    delay(1);
    if (iCurCntL > iTargetCntFlag) {
      if (sTargetSpeed > sAccSpeedL) {
        sAccSpeedL = sAccSpeedL + bAccPower;
        sAccSpeedR = sAccSpeedR + bAccPower;
        OzBot.SetStraight(sAccSpeedL, sAccSpeedR);
        iTargetCntFlag = iCurCntL + bAccFreq;
      }
    }
  } 
  while (iCurCntL <= iTarCntL);

  do {
    OzBot.GetCurSecTACH(iCurCntL, iCurCntR);
    delay(1);
    if (iCurCntL > iTargetCntFlag) {
      if (sAccSpeedL > CEN_SPEED_L) {
        sAccSpeedL = sAccSpeedL - bStopPower;
        sAccSpeedR = sAccSpeedR - bStopPower;
        OzBot.SetStraight(sAccSpeedL, sAccSpeedR);
        iTargetCntFlag = iCurCntL + bStopFreq;
      }
    }
  } 
  while ((iCurCntL<=iStopLen2) && (sAccSpeedL > CEN_SPEED_L) && (iCurCntL >= 50));

}
void AccMode(void)
{
  do {
    OzBot.GetSecCnt(bCnt);

    if (bNxtCnt == bCnt) {
      bNxtCnt = bCnt + 1;
      if (iStopLen > 0) {
        OzBot.SetStraight(CEN_SPEED_L, CEN_SPEED_R);
        delay(Rot_Time);
        CheckCount(iStopLen);
        OzBot.SetStraight(0, 0);
        delay(HOLD_STOP);
        OzBot.SetStraight(CEN_SPEED_L, CEN_SPEED_R);
        bNxtCnt = bPassFlag + 1;
      }
      CheckNext();
    }
    OzBot.GetRecStatus(bStatus);
  } 
  while (bStatus != 0);
}

void CalDeadZone(void)
{
  iSecLenL = CAL_DZ_SPEED_L + DeadZoneL;
  iSecLenR = CAL_DZ_SPEED_R + DeadZoneR;
  OzBot.ForwardLR(iSecLenL, iSecLenR);
  delay(500);
  OzBot.SetTachInLR(0, 0);
  delay(2000);
  OzBot.TachInDual(iSecLenL, iSecLenR);
  OzBot.StopDual();
  iSecLenR = (iSecLenL - iSecLenR) / 16;
  if (iSecLenR > 80) {
    iSecLenR = 80;
  }
  else if (iSecLenR < -80) {
    iSecLenR = -80;
  }
  if ( iSecLenR >= 0) {
    DeadZoneR = DeadZoneR + iSecLenR;
  }
  else {
    DeadZoneL = DeadZoneL - iSecLenR;
  }
  if ( DeadZoneL >= DeadZoneR) {
    DeadZoneL = DeadZoneL - DeadZoneR;
    DeadZoneR = 0;
  }
  else {
    DeadZoneR = DeadZoneR - DeadZoneL;
    DeadZoneL = 0;
  }
  OzBot.SetMotorDeadZone(DeadZoneL, DeadZoneR);

  OzBot.CursorRC(1, 1);
  OzBot.Display(DeadZoneL);
  OzBot.ClearEOL();
  OzBot.CursorRC(2, 1);
  OzBot.Display(DeadZoneR);
  OzBot.ClearEOL();
}
void CalMode(void)
{
  CheckAnalogIR();
  delay(2000);
  OzBot.IrCal(1);

  OzBot.SetVelLR(CAL_SPEED_L,-CAL_SPEED_R);
  delay(500);
  CheckAnalogIR();
  delay(500);
  CheckAnalogIR();
  OzBot.StopDual();	
  delay(300);

  OzBot.SetVelLR(-CAL_SPEED_L,CAL_SPEED_R);
  delay(500);
  CheckAnalogIR();
  delay(500);	
  CheckAnalogIR();
  OzBot.StopDual();	
  delay(300);

  OzBot.SetVelLR(CAL_SPEED_L,-CAL_SPEED_R);
  delay(500);
  CheckAnalogIR();
  delay(500);
  CheckAnalogIR();
  OzBot.StopDual();	
  delay(300);

  OzBot.SetVelLR(-CAL_SPEED_L,CAL_SPEED_R);
  delay(500);
  CheckAnalogIR();
  delay(500);	
  CheckAnalogIR();
  OzBot.StopDual();	
  delay(300);
}
void CheckLine(unsigned char bCheckIR)
{
  do {
    OzBot.GetIr(bIR);
  } 
  while (!(bIR & bCheckIR));
  OzBot.Beep();
  delay(2000);
}

void CheckAnalogIR(void)
{
  uint16_t sIR;
  do {
    OzBot.GetAnalogIr(2, sIR);
  } 
  while (sIR <= 180);
}
//------------------------------------------------------------------------
//	�主程式�
//------------------------------------------------------------------------
void setup(void)
{
  pinMode(8, INPUT);
  pinMode(9, INPUT);
  digitalWrite(8, HIGH);
  digitalWrite(9, HIGH);
  pinMode(12, OUTPUT);
  pinMode(13, OUTPUT);

  OzBot.SetP(PID_VAL_P);
  OzBot.SetI(PID_VAL_I);
  OzBot.SetD(PID_VAL_D);
  OzBot.SetScalar(PID_SCALAR);

  OzBot.SetErrScale(ERROR_VAL_1, ERROR_VAL_2, ERROR_VAL_3, ERROR_VAL_4);
  OzBot.SetSpdCtrlL(MIN_SPEED_L, MAX_SPEED_L);
  OzBot.SetSpdCtrlR(MIN_SPEED_R, MAX_SPEED_R);
  OzBot.SetCrossCount(CROSS_COUNT);
  OzBot.SetLineColor(0);
  OzBot.SetIrMode(1);

  OzBot.RecTurnControlL(18, -REC_SPEED_L, REC_SPEED_R, 15, 120)	;
  OzBot.RecTurnControlR( 2,  REC_SPEED_L, -REC_SPEED_R, 15, 120);
  OzBot.ArcTurnControlL(18, 			0, ROT_SPEED_R, 5, 120);
  OzBot.ArcTurnControlR( 2, ROT_SPEED_L, 0		  , 5, 120);
  OzBot.GetMotorDeadZone(DeadZoneL, DeadZoneR);
}
void loop(void)
{
  if (digitalRead(9) == LOW) {
    OzBot.Beep();
    digitalWrite(12, HIGH);
    delay(1500);
    if (digitalRead(9) == 0) {
      OzBot.Beep();
      delay(1500);
      if (digitalRead(9) == 0) {
        // DeadZone CAL
        OzBot.Beep();
        delay(1000);
        CalDeadZone();
      }
      else {
        // IR CAL
        CalMode();
      }
    }
    else {
      //REC_ROUTE:
      OzBot.StartRec(1);
      OzBot.AutoBeep(1);
      OzBot.SearchPath(1);
      OzBot.SetStraight(REC_SPEED_L, REC_SPEED_R);
      OzBot.SpdCtrlOn(1);
      do {
        OzBot.GetRecStatus(bStatus);
      } 
      while (bStatus != 0);
      OzBot.ForwardDual(REC_SPEED_L);
      delay(BRAKE_TIME);
      OzBot.BrakeDual();
      OzBot.AutoBeep(0);
      OzBot.StopRec();
    }
    digitalWrite(12, LOW);
  }
  else if (digitalRead(8) == LOW) {
    //RACE_ROUTE:
    CheckLine(4);
    bNxtCnt = 0;
    CheckNext();
    digitalWrite(13, HIGH);
    OzBot.Beep();
    delay(2000);
    OzBot.AutoBeep(0);
    OzBot.SetStraight(CEN_SPEED_L, CEN_SPEED_R);
    OzBot.SpdCtrlOn(2);
    AccMode();
    OzBot.ForwardDual(CEN_SPEED_L);
    delay(BRAKE_TIME);
    OzBot.BrakeDual();
    digitalWrite(13, LOW);
  }
}

