
#include "OzBot.h"
#include <SPI.h>
const int chipSelectPin = 11;

mOzBot::mOzBot()
{
  SPI.begin();
  SPI.setDataMode(SPI_MODE2);
  SPI.setClockDivider(SPI_CLOCK_DIV8);
  SPI.end();    
  pinMode(MISO, INPUT);
  pinMode(MOSI, INPUT);
  pinMode(SCK, INPUT);    
  digitalWrite(MISO, LOW);
  digitalWrite(MOSI, LOW);
  digitalWrite(SCK, LOW);
     
  digitalWrite(SS, HIGH);
  pinMode(SS, OUTPUT);
    
  digitalWrite(chipSelectPin, HIGH);    
  pinMode(chipSelectPin, OUTPUT);
  pinMode(SCK, OUTPUT);
  delayMicroseconds(10);
  pinMode(SCK, INPUT);
  delayMicroseconds(10000);
  
}
unsigned char mOzBot::writeRegister(unsigned char Len, unsigned char Skip) {

  unsigned char *sptr, Checksum, Data;
  int i, k;
  digitalWrite(MOSI, HIGH);
  digitalWrite(SCK, HIGH);    
  SPI.begin();
  SPI.setDataMode(SPI_MODE2);
  SPI.setClockDivider(SPI_CLOCK_DIV8);
  
  sptr = Cmd;
  Checksum = 0;
  for (i = 0 ; i < Len ; i++) {
    Checksum += *sptr++;
  }
  *sptr = -Checksum;
  for(k = 0 ; k < kRetry ; k++) {
    digitalWrite(chipSelectPin, LOW);
    delayMicroseconds(kDelay);
    if(SPI.transfer(0x55) == 0xFF) {
        delayMicroseconds(kDelay);
        if(SPI.transfer(0x33) == 0xFF) {
            delayMicroseconds(kDelay);  
            if(SPI.transfer(Cmd[0]) == 0xA5) {        
              sptr = &Cmd[1];
              for (i = 1 ; i <= Len ; i++) {
                delayMicroseconds(kDelay);
                SPI.transfer(*sptr++);        
              }
              if(Skip == 0) {
                  digitalWrite(chipSelectPin, HIGH);
                  pinMode(MOSI, INPUT);
                  pinMode(SCK, INPUT);    
                  digitalWrite(MOSI, LOW);
                  digitalWrite(SCK, LOW);               
                  SPI.end();
              }
              return 1;
            }
        }
    }
    digitalWrite(chipSelectPin, HIGH);
    delayMicroseconds(kCmdDelay);
  }
  if(Skip == 0) {
      pinMode(MOSI, INPUT);
      pinMode(SCK, INPUT);    
      digitalWrite(MOSI, LOW);
      digitalWrite(SCK, LOW);   
      SPI.end();
  }   
  return 0;

}


unsigned char mOzBot::readRegister(unsigned char CmdLen, unsigned char Len) {

  unsigned char *sptr, Checksum, Data;
  int i, k;
  
  if (writeRegister(CmdLen, 1)) {
    for(k = 0 ; k < kRetry ; k++) {
      delayMicroseconds(kCmdDelay);    
      digitalWrite(chipSelectPin, LOW);
      delayMicroseconds(kDelay);
      if(SPI.transfer(0x55) == 0xFF) {
        delayMicroseconds(kDelay);      
        if(SPI.transfer(0x33) == 0xFF) {
            delayMicroseconds(kDelay);          
            if(SPI.transfer(0x15) == 0xA5) {
              Checksum = 0;                    
              sptr = DataBuf;               
              for(i = 0 ; i <= Len ; i++) {
                delayMicroseconds(kDelay);
                Data = SPI.transfer(0xFF);
                *sptr++ = Data;
                Checksum += Data;                                                       
              }
              if(Checksum == 0xFF) {
                digitalWrite(chipSelectPin, HIGH);
                pinMode(SCK, INPUT);                
                pinMode(MOSI, INPUT);    
                digitalWrite(MOSI, LOW);
                digitalWrite(SCK, LOW);                 
                SPI.end();                
                return 1;                 
              }         
            }
        }
      }
      digitalWrite(chipSelectPin, HIGH);
    }
  }
  pinMode(MOSI, INPUT);
  pinMode(SCK, INPUT);    
  digitalWrite(MOSI, LOW);
  digitalWrite(SCK, LOW);
  SPI.end();         
  return 0;

}
void mOzBot::SetP(unsigned char P)
{
  Cmd[0] = 131;
  Cmd[1] = P;
  writeRegister(2);
}
void mOzBot::SetI(unsigned char I)
{
  Cmd[0] = 133;
  Cmd[1] = I;
  writeRegister(2);
}
void mOzBot::SetD(unsigned char D)
{
  Cmd[0] = 135;
  Cmd[1] = D;
  writeRegister(2);
}
void mOzBot::SetScalar(unsigned char Scalar)
{
  Cmd[0] = 137;
  Cmd[1] = Scalar;
  writeRegister(2);
}
void mOzBot::SetIrMode(uint8_t Mode)
{
  Cmd[0] = 154;
  Cmd[1] = Mode;
  writeRegister(2);
}
void mOzBot::SetErrScale(unsigned char Err1, unsigned char Err2, unsigned char Err3, unsigned char Err4)
{
  Cmd[0] = 164;
  Cmd[1] = Err1;
  Cmd[2] = Err2;
  Cmd[3] = Err3;
  Cmd[4] = Err4;
  writeRegister(5);

}
void mOzBot::SetSpdCtrlL(int16_t L, int16_t H)
{
  Cmd[0] = 139;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  Cmd[3] = H;
  Cmd[4] = H >> 8;
  writeRegister(5);
}
void mOzBot::SetSpdCtrlR(int16_t L, int16_t H)
{
  Cmd[0] = 157;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  Cmd[3] = H;
  Cmd[4] = H >> 8;
  writeRegister(5);
}
void mOzBot::SetCrossCount(uint16_t Cnt)
{
  Cmd[0] = 178;
  Cmd[1] = Cnt;
  Cmd[2] = Cnt >> 8;
  writeRegister(3);
}
void mOzBot::SetMotorDeadZone(int16_t L, int16_t R)
{
  Cmd[0] = 123;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  Cmd[3] = R;
  Cmd[4] = R >> 8;
  writeRegister(5);
}
void mOzBot::GetMotorDeadZone(int16_t& L, int16_t& R)
{
  Cmd[0] = 122;
  if(readRegister(1, 4)) {
    L = DataBuf[0] | (DataBuf[1] << 8);
    R = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::Beep(void)
{
  Cmd[0] = 127;
  writeRegister(1);
}
void mOzBot::AutoBeep(uint8_t Beep)
{
  Cmd[0] = 128;
  Cmd[1] = Beep;
  writeRegister(2);
}

void mOzBot::SetStraight(int16_t L, int16_t R)
{
  Cmd[0] = 162;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  Cmd[3] = R;
  Cmd[4] = R >> 8;
  writeRegister(5);
}
void mOzBot::SpdCtrlOn(uint8_t Mode)
{
  Cmd[0] = 141;
  Cmd[1] = Mode;
  writeRegister(2);
}
void mOzBot::SpdCtrlOff(void)
{
  Cmd[0] = 142;
  writeRegister(1);
}
void mOzBot::StartRec(uint8_t Rec)
{
  Cmd[0] = 72;
  Cmd[1] = Rec;
  writeRegister(2);
}
void mOzBot::StopRec(void)
{
  Cmd[0] = 73;
  writeRegister(1);
}
void mOzBot::SearchPath(uint8_t Hand)
{
  Cmd[0] = 60;
  Cmd[1] = Hand;
  writeRegister(2);
}
void mOzBot::GetSecCnt(uint8_t& Param0)
{
  Cmd[0] = 70;
  if (readRegister(1, 1)) {
    Param0 = DataBuf[0];
  }
}

void mOzBot::GetOptiPathCnt(uint8_t& Param0)
{
  Cmd[0] = 67;
  if (readRegister(1, 1)) {
    Param0 = DataBuf[0];
  }
}
void mOzBot::TotalPath(uint8_t Param0, uint8_t& Param1, uint8_t& Param2, uint8_t& Param3, int16_t& Param4, int16_t& Param5)
{
  Cmd[0] = 61;
  Cmd[1] = Param0;  
  if (readRegister(2, 7)) {
    Param1 = DataBuf[0];
    Param2 = DataBuf[1];
    Param3 = DataBuf[2];
    Param4 = DataBuf[3] | (DataBuf[4]<<8);
    Param5 = DataBuf[5] | (DataBuf[6]<<8);
  }
}
void mOzBot::OptimumPath(uint8_t Param0, uint8_t& Param1, uint8_t& Param2, uint8_t& Param3, int16_t& Param4, int16_t& Param5)
{
  Cmd[0] = 62;
  Cmd[1] = Param0;  
  if (readRegister(2, 7)) {
    Param1 = DataBuf[0];
    Param2 = DataBuf[1];
    Param3 = DataBuf[2];
    Param4 = DataBuf[3] | (DataBuf[4]<<8);
    Param5 = DataBuf[5] | (DataBuf[6]<<8);
  }
}
void mOzBot::GetRecStatus(uint8_t &Status)
{
  Cmd[0] = 69;
  if (readRegister(1, 1)) {
    Status = DataBuf[0];
  }
}
void mOzBot::ForwardDual(uint16_t Speed)
{
  Cmd[0] = 91;
  Cmd[1] = Speed;
  Cmd[2] = Speed >> 8;
  writeRegister(3);
}
void mOzBot::BackwardDual(uint16_t Speed)
{
  Cmd[0] = 95;
  Cmd[1] = Speed;
  Cmd[2] = Speed >> 8;
  writeRegister(3);
}
void mOzBot::StopDual(void)
{
  Cmd[0] = 98;
  writeRegister(1);
}
void mOzBot::BrakeDual(void)
{
  Cmd[0] = 101;
  writeRegister(1);
}
void mOzBot::ForwardL(uint16_t L)
{
  Cmd[0] = 88;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  writeRegister(3);
}
void mOzBot::ForwardR(uint16_t R)
{
  Cmd[0] = 89;
  Cmd[1] = R;
  Cmd[2] = R >> 8;
  writeRegister(3);
}
void mOzBot::ForwardLR(uint16_t L, uint16_t R)
{
  Cmd[0] = 90;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  Cmd[3] = R;
  Cmd[4] = R >> 8;
  writeRegister(5);
}
void mOzBot::BackwardL(uint16_t L)
{
  Cmd[0] = 92;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  writeRegister(3);
}
void mOzBot::BackwardR(uint16_t R)
{
  Cmd[0] = 93;
  Cmd[1] = R;
  Cmd[2] = R >> 8;
  writeRegister(3);
}
void mOzBot::BackwardLR(uint16_t L, uint16_t R)
{
  Cmd[0] = 94;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  Cmd[3] = R;
  Cmd[4] = R >> 8;
  writeRegister(5);
}
void mOzBot::TachInL(int16_t& L)
{
  Cmd[0] = 172;
  if (readRegister(1, 2)) {
    L = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::TachInR(int16_t& R)
{
  Cmd[0] = 173;
  if (readRegister(1, 2)) {
    R = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::TachInDual(int16_t& L, int16_t& R)
{
  Cmd[0] = 174;
  if (readRegister(1, 4)) {
    L = DataBuf[0] | (DataBuf[1] << 8);
    R = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::SetTachInLR(int16_t L, int16_t R)
{
  Cmd[0] = 177;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  Cmd[3] = R;
  Cmd[4] = R >> 8;
  writeRegister(5);
}
void mOzBot::SetVelLR(int16_t L, int16_t R)
{
  Cmd[0] = 118;
  Cmd[1] = L;
  Cmd[2] = L >> 8;
  Cmd[3] = R;
  Cmd[4] = R >> 8;
  writeRegister(5);
}
void mOzBot::IrCal(uint8_t Time)
{
  Cmd[0] = 146;
  Cmd[1] = Time;
  writeRegister(2);
}

void mOzBot::SetLineColor(uint8_t Color)
{
  Cmd[0] = 170;
  Cmd[1] = Color;
  writeRegister(2);
}
void mOzBot::GetCurSecTACH(int16_t& L, int16_t& R)
{
  Cmd[0] = 71;
  if (readRegister(1, 4)) {
    L = DataBuf[0] | (DataBuf[1] << 8);
    R = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::GetSecLen(uint8_t No, int16_t& L, int16_t& R)
{
  Cmd[0] = 74;
  Cmd[1] = No;
  if (readRegister(2, 4)) {
    L = DataBuf[0] | (DataBuf[1] << 8);
    R = DataBuf[2] | (DataBuf[3] << 8);
  }
}

void mOzBot::GetSecDir(uint8_t No, uint8_t& Dir)
{
  Cmd[0] = 75;
  Cmd[1] = No;
  if (readRegister(2, 1)) {
    Dir = DataBuf[0] ;
  }
}
void mOzBot::ArcTurnControlL(uint8_t TurnState, int16_t LeftSpeed, int16_t RightSpeed, int16_t BlackCount, int16_t DelayCount)
{
  Cmd[0] = 76;
  Cmd[1] = TurnState;
  Cmd[2] = LeftSpeed;
  Cmd[3] = LeftSpeed >> 8;
  Cmd[4] = RightSpeed;
  Cmd[5] = RightSpeed >> 8;
  Cmd[6] = BlackCount;
  Cmd[7] = BlackCount >> 8;
  Cmd[8] = DelayCount;
  Cmd[9] = DelayCount >> 8;
  writeRegister(10);
}
void mOzBot::ArcTurnControlR(uint8_t TurnState, int16_t LeftSpeed, int16_t RightSpeed, int16_t BlackCount, int16_t DelayCount)
{
  Cmd[0] = 77;
  Cmd[1] = TurnState;
  Cmd[2] = LeftSpeed;
  Cmd[3] = LeftSpeed >> 8;
  Cmd[4] = RightSpeed;
  Cmd[5] = RightSpeed >> 8;
  Cmd[6] = BlackCount;
  Cmd[7] = BlackCount >> 8;
  Cmd[8] = DelayCount;
  Cmd[9] = DelayCount >> 8;
  writeRegister(10);
}
void mOzBot::RecTurnControlL(uint8_t TurnState, int16_t LeftSpeed, int16_t RightSpeed, int16_t BlackCount, int16_t DelayCount)
{
  Cmd[0] = 78;
  Cmd[1] = TurnState;
  Cmd[2] = LeftSpeed;
  Cmd[3] = LeftSpeed >> 8;
  Cmd[4] = RightSpeed;
  Cmd[5] = RightSpeed >> 8;
  Cmd[6] = BlackCount;
  Cmd[7] = BlackCount >> 8;
  Cmd[8] = DelayCount;
  Cmd[9] = DelayCount >> 8;
  writeRegister(10);
}
void mOzBot::RecTurnControlR(uint8_t TurnState, int16_t LeftSpeed, int16_t RightSpeed, int16_t BlackCount, int16_t DelayCount)
{
  Cmd[0] = 79;
  Cmd[1] = TurnState;
  Cmd[2] = LeftSpeed;
  Cmd[3] = LeftSpeed >> 8;
  Cmd[4] = RightSpeed;
  Cmd[5] = RightSpeed >> 8;
  Cmd[6] = BlackCount;
  Cmd[7] = BlackCount >> 8;
  Cmd[8] = DelayCount;
  Cmd[9] = DelayCount >> 8;
  writeRegister(10);
}
void mOzBot::Display(uint8_t Val) {
  Cmd[0] = 237;
  Cmd[1] = 0;
  Cmd[2] = Val;
  writeRegister(3);
}
void mOzBot::Display(int8_t Val) {
  Cmd[0] = 238;
  Cmd[1] = 0;
  Cmd[2] = Val;
  writeRegister(3);
}
void mOzBot::Display(uint16_t Val) {
  Cmd[0] = 239;
  Cmd[1] = 0;
  Cmd[2] = Val;
  Cmd[3] = Val>>8;    
  writeRegister(4);
}
void mOzBot::Display(int16_t Val) {
  Cmd[0] = 240;
  Cmd[1] = 0;
  Cmd[2] = Val;
  Cmd[3] = Val>>8;  
  writeRegister(4);
}
void mOzBot::DisplayChar(uint8_t Val) {
  Cmd[0] = 235;
  Cmd[1] = Val;
  writeRegister(2);
}
void mOzBot::BackLightOn(uint8_t Time)
{
  Cmd[0] = 227;
  Cmd[1] = Time;
  writeRegister(2);
}
void mOzBot::BackLightOff(void)
{
  Cmd[0] = 228;
  writeRegister(1);
}
void mOzBot::DisplayOn(void)
{
  Cmd[0] = 229;
  writeRegister(1);
}
void mOzBot::DisplayOff(void)
{
  Cmd[0] = 230;
  writeRegister(1);
} 
void mOzBot::CursorOn(void)
{
  Cmd[0] = 231;
  writeRegister(1);
}
void mOzBot::CursorOff(void)
{
  Cmd[0] = 232;
  writeRegister(1);
} 
void mOzBot::Clear(void)
{
  Cmd[0] = 210;
  writeRegister(1);
}
void mOzBot::Home(void)
{
  Cmd[0] = 211;
  writeRegister(1);
}
void mOzBot::ShowString(char *ptr)
{
  for(int i = 0 ; i < 8 ; i++) {
    if(*ptr == 0) break;
    DisplayChar(*ptr++);
  }
}
void mOzBot::Display(char *ptr)
{
  for(int i = 0 ; i < 8 ; i++) {
    if(*ptr == 0) break;
    DisplayChar(*ptr++);
  }
}
void mOzBot::ClearEOL(void)
{
  Cmd[0] = 221;
  writeRegister(1);
}
void mOzBot::CR(void)
{
  Cmd[0] = 220;
  writeRegister(1);
}
void mOzBot::CursorRC(uint8_t R, uint8_t C) 
{
  Cmd[0] = 223;
  Cmd[1] = C;
  Cmd[2] = R;  
  writeRegister(3);
}
void mOzBot::CursorRow(uint8_t R) 
{
  Cmd[0] = 225;
  Cmd[1] = R;  
  writeRegister(2);
}
void mOzBot::CursorCol(uint8_t C) 
{
  Cmd[0] = 224;
  Cmd[1] = C;
  writeRegister(2);
}
void mOzBot::GetIr(uint8_t& bIr)
{
  Cmd[0] = 130;
  if (readRegister(1, 1)) {
    bIr = DataBuf[0];
  }  
}
void mOzBot::GetAnalogIr(uint8_t Ch, uint16_t& Value)
{
  Cmd[0] = 156;
  Cmd[1] = Ch;
  if (readRegister(2, 2)) {
    Value = DataBuf[0] | (DataBuf[1] << 8);
  }  
}
unsigned char mOzBot::CheckLowBattery(void)
{
  Cmd[0] = 126;
  if (readRegister(1, 1)) {
    return DataBuf[0];
  }  
  return 1;
}
void mOzBot::LowBatteryAlarmOn(void)
{
  Cmd[0] = 124;
  writeRegister(1);  
}
void mOzBot::LowBatteryAlarmOff(void)
{
  Cmd[0] = 125;
  writeRegister(1);  
}
void mOzBot::SetAccMode(uint8_t Mode)
{
  Cmd[0] = 111;
  Cmd[1] = Mode;
  writeRegister(2); 
}
void mOzBot::SetGyroMode(uint8_t Mode)
{
  Cmd[0] = 113;
  Cmd[1] = Mode;
  writeRegister(2); 
}
void mOzBot::GetAccX(int16_t& Param0)
{
  Cmd[0] = 105;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetAccY(int16_t& Param0)
{
  Cmd[0] = 106;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetAccZ(int16_t& Param0)
{
  Cmd[0] = 107;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetAccXYZ(int16_t& Param0, int16_t& Param1, int16_t& Param2)
{
  Cmd[0] = 112;
  if (readRegister(1, 6)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
    Param1 = DataBuf[2] | (DataBuf[3] << 8);
    Param2 = DataBuf[4] | (DataBuf[5] << 8);
  }
}
void mOzBot::GetGyroX(int16_t& Param0)
{
  Cmd[0] = 108;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetGyroY(int16_t& Param0)
{
  Cmd[0] = 109;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetGyroZ(int16_t& Param0)
{
  Cmd[0] = 110;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetGyroXYZ(int16_t& Param0, int16_t& Param1, int16_t& Param2)
{
  Cmd[0] = 114;
  if (readRegister(1, 6)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
    Param1 = DataBuf[2] | (DataBuf[3] << 8);
    Param2 = DataBuf[4] | (DataBuf[5] << 8);
  }
}
void mOzBot::GetGyroXADVal(uint16_t& Param0)
{
  Cmd[0] = 52;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetGyroYADVal(uint16_t& Param0)
{
  Cmd[0] = 53;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetGyroZADVal(uint16_t& Param0)
{
  Cmd[0] = 54;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
} 
void mOzBot::GetAccMaxMinX(int16_t& Param0, int16_t& Param1)
{
  Cmd[0] = 43;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
    Param1 = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::GetAccMaxMinY(int16_t& Param0, int16_t& Param1)
{
  Cmd[0] = 44;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
    Param1 = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::GetAccMaxMinZ(int16_t& Param0, int16_t& Param1)
{
  Cmd[0] = 45;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
    Param1 = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::GetGyroMaxMinX(int16_t& Param0, int16_t& Param1)
{
  Cmd[0] = 49;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
    Param1 = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::GetGyroMaxMinY(int16_t& Param0, int16_t& Param1)
{
  Cmd[0] = 50;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
    Param1 = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::GetGyroMaxMinZ(int16_t& Param0, int16_t& Param1)
{
  Cmd[0] = 51;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
    Param1 = DataBuf[2] | (DataBuf[3] << 8);
  }
}
void mOzBot::GetGyroIntegX(int32_t& Param0)
{
  Cmd[0] = 46;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8)| (DataBuf[2]<<16) | (DataBuf[3] << 24);
  }
}                                                                                                                                  
void mOzBot::GetGyroIntegY(int32_t& Param0)
{
  Cmd[0] = 47;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8)| (DataBuf[2]<<16) | (DataBuf[3] << 24);
  }
}                                                                                                                                                            
void mOzBot::GetGyroIntegZ(int32_t& Param0)
{
  Cmd[0] = 48;
  if (readRegister(1, 4)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8)| (DataBuf[2]<<16) | (DataBuf[3] << 24);
  }
}       

void mOzBot::GetCompDevAngle(uint16_t & Param0)  
{
  Cmd[0] = 189;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}       
void mOzBot::SetCompTargetAngle(uint16_t Param0)
{
  Cmd[0] = 205;
  Cmd[1] = Param0;
  Cmd[2] = Param0>>8;  
  writeRegister(3);
}
void mOzBot::SetCompCurrentTargetAngle(void)
{
  Cmd[0] = 204;
  writeRegister(1);
}
void mOzBot::CompassCal(uint8_t Param0)
{
  Cmd[0] = 196;
  Cmd[1] = Param0;
  writeRegister(2);
}
uint8_t mOzBot::GetCompass(uint16_t& Param0)
{
  Cmd[0] = 188;
  if (readRegister(1, 3)) {
    Param0 = DataBuf[1] | (DataBuf[2] << 8);
    return DataBuf[0];
  } 
  return 0;
}
void mOzBot::GetCompX(int16_t& Param0)
{
  Cmd[0] = 102;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetCompY(int16_t& Param0)
{
  Cmd[0] = 103;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::GetCompZ(int16_t& Param0)
{
  Cmd[0] = 104;
  if (readRegister(1, 2)) {
    Param0 = DataBuf[0] | (DataBuf[1] << 8);
  }
}
void mOzBot::SetCtrlFreq(unsigned char Param0)
{
  Cmd[0] = 159;
  Cmd[1] = Param0;  
  writeRegister(2);

} 
void mOzBot::GetTotalPathCnt(unsigned char& Param0)
{
  Cmd[0] = 66;
  if (readRegister(1, 1)) {
    Param0 = DataBuf[0];
  }
}
void mOzBot::Ranging(uint8_t Param0)
{
  Cmd[0] = 245;
  Cmd[1] = Param0;  
  writeRegister(2);

} 
void mOzBot::RepeatRanging(void)
{
  Cmd[0] = 246;
  writeRegister(1);

} 
void mOzBot::StopRanging(void)
{
  Cmd[0] = 247; 
  writeRegister(1);
}
void mOzBot::AccCal(void)
{
  Cmd[0] = 41; 
  writeRegister(1);

} 
void mOzBot::GyroCal(void)
{
  Cmd[0] = 42; 
  writeRegister(1);

}  
void mOzBot::GetDistance(uint8_t& Param0, uint8_t& Param1, uint8_t& Param2, uint8_t& Param3, uint8_t& Param4)
{
  Cmd[0] = 250;
  if (readRegister(1, 5)) {
    Param0 = DataBuf[0];
    Param1 = DataBuf[1];
    Param2 = DataBuf[2];
    Param3 = DataBuf[3];
    Param4 = DataBuf[4];
  }

}
void mOzBot::GetIrCal(unsigned char Param0, uint16_t& Param1, uint16_t& Param2)
{
  Cmd[0] = 148;
  Cmd[1] = Param0;  
  if (readRegister(2, 4)) {
    Param1 = DataBuf[0] | (DataBuf[1] << 8);
    Param2 = DataBuf[2] | (DataBuf[3] << 8);
  }

}
void mOzBot::SetIrThreshold(uint8_t Param0)
{
  Cmd[0] = 150; 
  Cmd[1] = Param0;
  writeRegister(2);

}                         
mOzBot OzBot;

