/**
  ******************************************************************************
  * @file 		GamepadBT
  * @author  	Innovati Team
  * @version  	V0.1.0
  * @date     	07/18/2011
  * @brief  	Main program body
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, Innovati SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 Innovati, Inc</center></h2>
  */

/* Includes ------------------------------------------------------------------*/
#include "arm32f.h"
#include "I2CRoutines.h"
#include "GamepadBT.h"
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
/*----------------------command list------------------------------------------*/                                                                
/*----------------------------------------------------------------------------*/
GamepadBT::GamepadBT(uint8_t mySlaveID)
{
	SlaveID = mySlaveID;
#ifdef __CHECK__
	char *ptr = (char *)0x1f0007B7;
	if((mySlaveID & 1) && (*ptr != 'H')) return;
#endif	
  InitArmCommander();
}
void GamepadBT::GetLXYPos(char &LXPos, char &LYPos)
{
  __innoCommandBuff__[0] = 113;
  __innoNumByteToRead__ = 3;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	  LXPos = (int8_t)__innoCommandBuff__[0];
	  LYPos = (int8_t)__innoCommandBuff__[1];
  }	        	  	
}
void GamepadBT::GetLXYPos(int8_t &LXPos, int8_t &LYPos)
{
  __innoCommandBuff__[0] = 113;
  __innoNumByteToRead__ = 3;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	  LXPos = (int8_t)__innoCommandBuff__[0];
	  LYPos = (int8_t)__innoCommandBuff__[1];
  }	        	  	
}
void GamepadBT::GetRXYPos(char &RXPos, char &RYPos)
{
  __innoCommandBuff__[0] = 114;
  __innoNumByteToRead__ = 3;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	RXPos = (int8_t)__innoCommandBuff__[0];
  	RYPos = (int8_t)__innoCommandBuff__[1];
  }		     
} 
void GamepadBT::GetRXYPos(int8_t &RXPos, int8_t &RYPos)
{
  __innoCommandBuff__[0] = 114;
  __innoNumByteToRead__ = 3;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	RXPos = (int8_t)__innoCommandBuff__[0];
  	RYPos = (int8_t)__innoCommandBuff__[1];
  }		     
}
void GamepadBT::GetL4WayValue(uint8_t &Way)
{
  __innoCommandBuff__[0] = 115;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  	Way = __innoCommandBuff__[0];
}
void GamepadBT::GetR4WayValue(uint8_t &Way)
{
  __innoCommandBuff__[0] = 116;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  	Way = __innoCommandBuff__[0];
}
void GamepadBT::GetL8WayValue(uint8_t &Way)
{
  __innoCommandBuff__[0] = 117;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  	Way = __innoCommandBuff__[0];
}
void GamepadBT::GetR8WayValue(uint8_t &Way)
{
  __innoCommandBuff__[0] = 118;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__)) 
  	Way = __innoCommandBuff__[0];
}
uint16_t GamepadBT::GetKeyStatus(void)
{
  __innoCommandBuff__[0] = 138;
  __innoNumByteToRead__ = 3;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__)) return(*((uint16_t *)&__innoCommandBuff__[0]));
  else                                                          return 0;
}
void GamepadBT::GetDir4Way(uint8_t &Dir4Way)
{
  __innoCommandBuff__[0] = 141;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
    Dir4Way = __innoCommandBuff__[0];
}
void GamepadBT::GetDir8Way(uint8_t &Dir8Way)
{
  __innoCommandBuff__[0] = 142;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  	Dir8Way = __innoCommandBuff__[0];
}

void GamepadBT::SetStickRefreshRate(uint8_t Rate)
{   
  __innoCommandBuff__[0] = 111;
  __innoCommandBuff__[2] = Rate;
  __i2cWriteBuffer(__innoCommandBuff__, SlaveID, 5);
}
void GamepadBT::GetStickRefreshRate(uint8_t& Rate)
{
  __innoCommandBuff__[0] = 112;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	  Rate = __innoCommandBuff__[0];
  }	        
}
void GamepadBT::SetKeyRepeatFunc(uint16_t KeyID)
{    
  __innoCommandBuff__[0] = 133;
  *((uint16_t *)&__innoCommandBuff__[2]) = KeyID;
  __i2cWriteBuffer(__innoCommandBuff__, SlaveID, 6);
}
void GamepadBT::SetRepeatTime(uint8_t Time)
{
  __innoCommandBuff__[0] = 134;
  __innoCommandBuff__[2] = Time;
  __i2cWriteBuffer(__innoCommandBuff__, SlaveID, 5);
}
void GamepadBT::GetRepeatTime(uint8_t& Time)
{
  __innoCommandBuff__[0] = 135;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	  Time = __innoCommandBuff__[0];
  }	        
}
void GamepadBT::SetRepeatRate(uint8_t Rate)
{    
  __innoCommandBuff__[0] = 136;
  __innoCommandBuff__[2] = Rate;
  __i2cWriteBuffer(__innoCommandBuff__, SlaveID, 5);
}
void GamepadBT::GetRepeatRate(uint8_t& Rate)
{
  __innoCommandBuff__[0] = 137;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	  Rate = __innoCommandBuff__[0];
  }	        
}
void GamepadBT::GetKeyRepeatFunc(uint16_t& KeyID)
{
  __innoCommandBuff__[0] = 147;
  __innoNumByteToRead__ = 3;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	  KeyID = *((uint16_t *)&__innoCommandBuff__);   
  }	        
}
void GamepadBT::GetAnalog(uint8_t& Analog)
{
  __innoCommandBuff__[0] = 150;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	  Analog = __innoCommandBuff__[0];
  }	        
}
void GamepadBT::GetConnect(uint8_t& Connect)
{
  __innoCommandBuff__[0] = 151;
  __innoNumByteToRead__ = 2;
  if(!__i2cReadBuffer(__innoCommandBuff__, SlaveID, 3, &__innoNumByteToRead__))
  {
	  Connect = __innoCommandBuff__[0];
  }	        
}
